Linux Headquarters
[ Register ]
[ About us ] [ Home Page ]

Advertisement
[ Kernel ] [ Documentation ] [ Links ] [ Books ]

Advertisement

Kernel v2.6.24 /fs/ecryptfs/super.c

Filename:/fs/ecryptfs/super.c
Lines Added:27
Lines Deleted:11
Also changed in: (Previous) 2.6.24-rc8  2.6.24-rc7-git8  2.6.24-rc7-git7  2.6.24-rc7-git6  2.6.24-rc7-git5  2.6.24-rc7-git4 
(Following) 2.6.24-git16  2.6.24-git17  2.6.24-git18  2.6.24-git19  2.6.24-git20  2.6.24-git21 

Location
[  2.6.24
  [  fs
    [  ecryptfs
       o  super.c

Patch

diff --git a/fs/ecryptfs/super.c b/fs/ecryptfs/super.c
index 7b3f0cc..4859c4e 100644
--- a/fs/ecryptfs/super.c
+++ b/fs/ecryptfs/super.c
@@ -27,6 +27,7 @@
 #include <linux/mount.h>
 #include <linux/key.h>
 #include <linux/seq_file.h>
+#include <linux/file.h>
 #include <linux/crypto.h>
 #include "ecryptfs_kernel.h"
 
@@ -46,15 +47,16 @@ struct kmem_cache *ecryptfs_inode_info_cache;
  */
 static struct inode *ecryptfs_alloc_inode(struct super_block *sb)
 {
-   struct ecryptfs_inode_info *ecryptfs_inode;
+   struct ecryptfs_inode_info *inode_info;
    struct inode *inode = NULL;
 
-   ecryptfs_inode = kmem_cache_alloc(ecryptfs_inode_info_cache,
-                 GFP_KERNEL);
-   if (unlikely(!ecryptfs_inode))
+   inode_info = kmem_cache_alloc(ecryptfs_inode_info_cache, GFP_KERNEL);
+   if (unlikely(!inode_info))
       goto out;
-   ecryptfs_init_crypt_stat(&ecryptfs_inode->crypt_stat);
-   inode = &ecryptfs_inode->vfs_inode;
+   ecryptfs_init_crypt_stat(&inode_info->crypt_stat);
+   mutex_init(&inode_info->lower_file_mutex);
+   inode_info->lower_file = NULL;
+   inode = &inode_info->vfs_inode;
 out:
    return inode;
 }
@@ -63,9 +65,10 @@ out:
  * ecryptfs_destroy_inode
  * @inode: The ecryptfs inode
  *
  * This is used during the final destruction of the inode.
  * All allocation of memory related to the inode, including allocated
  * memory in the crypt_stat struct, will be released here.
  * There should be no chance that this deallocation will be missed.
  */
 static void ecryptfs_destroy_inode(struct inode *inode)
@@ -73,7 +76,20 @@ static void ecryptfs_destroy_inode(struct inode *inode)
    struct ecryptfs_inode_info *inode_info;
 
    inode_info = ecryptfs_inode_to_private(inode);
-   ecryptfs_destruct_crypt_stat(&inode_info->crypt_stat);
+   mutex_lock(&inode_info->lower_file_mutex);
+   if (inode_info->lower_file) {
+      struct dentry *lower_dentry =
+         inode_info->lower_file->f_dentry;
+
+      BUG_ON(!lower_dentry);
+      if (lower_dentry->d_inode) {
+         fput(inode_info->lower_file);
+         inode_info->lower_file = NULL;
+         d_drop(lower_dentry);
+      }
+   }
+   mutex_unlock(&inode_info->lower_file_mutex);
+   ecryptfs_destroy_crypt_stat(&inode_info->crypt_stat);
    kmem_cache_free(ecryptfs_inode_info_cache, inode_info);
 }
 
@@ -104,7 +120,7 @@ static void ecryptfs_put_super(struct super_block *sb)
 {
    struct ecryptfs_sb_info *sb_info = ecryptfs_superblock_to_private(sb);
 
-   ecryptfs_destruct_mount_crypt_stat(&sb_info->mount_crypt_stat);
+   ecryptfs_destroy_mount_crypt_stat(&sb_info->mount_crypt_stat);
    kmem_cache_free(ecryptfs_sb_info_cache, sb_info);
    ecryptfs_set_superblock_private(sb, NULL);
 }
@@ -63,9 +65,10 @@ out:
  * @inode: The ecryptfs inode
  *
- * This is used during the final destruction of the inode.
- * All allocation of memory related to the inode, including allocated
- * memory in the crypt_stat struct, will be released here.
+ * This is used during the final destruction of the inode.  All
+ * allocation of memory related to the inode, including allocated
+ * memory in the crypt_stat struct, will be released here. This
+ * function also fput()'s the persistent file for the lower inode.
  * There should be no chance that this deallocation will be missed.
  */


Comments: webmaster (at) linuxhq.com.
Advertising: banners (at) linuxhq.com.
Compilation ©1998-2008 Linux Headquarters, Inc.